/*
 * このソースコードは blanco Frameworkにより自動生成されました。
 */
package blanco.filemanager.task;

import java.io.IOException;

import blanco.filemanager.task.valueobject.BlancoFileManagerDbTableProcessInput;

/**
 * バッチ処理クラス [BlancoFileManagerDbTableBatchProcess]。
 *
 * <P>バッチ処理の呼び出し例。</P>
 * <code>
 * java -classpath (クラスパス) blanco.filemanager.task.BlancoFileManagerDbTableBatchProcess -help
 * </code>
 */
public class BlancoFileManagerDbTableBatchProcess {
    /**
     * 正常終了。
     */
    public static final int END_SUCCESS = 0;

    /**
     * 入力異常終了。内部的にjava.lang.IllegalArgumentExceptionが発生した場合。
     */
    public static final int END_ILLEGAL_ARGUMENT_EXCEPTION = 7;

    /**
     * 入出力例外終了。内部的にjava.io.IOExceptionが発生した場合。
     */
    public static final int END_IO_EXCEPTION = 8;

    /**
     * 異常終了。バッチの処理開始に失敗した場合、および内部的にjava.lang.Errorまたはjava.lang.RuntimeExceptionが発生した場合。
     */
    public static final int END_ERROR = 9;

    /**
     * コマンドラインから実行された際のエントリポイントです。
     *
     * @param args コンソールから引き継がれた引数。
     */
    public static final void main(final String[] args) {
        final BlancoFileManagerDbTableBatchProcess batchProcess = new BlancoFileManagerDbTableBatchProcess();

        // バッチ処理の引数。
        final BlancoFileManagerDbTableProcessInput input = new BlancoFileManagerDbTableProcessInput();

        boolean isNeedUsage = false;
        boolean isFieldJdbcdriverProcessed = false;
        boolean isFieldJdbcurlProcessed = false;
        boolean isFieldJdbcuserProcessed = false;
        boolean isFieldJdbcpasswordProcessed = false;

        // コマンドライン引数の解析をおこないます。
        for (int index = 0; index < args.length; index++) {
            String arg = args[index];
            if (arg.startsWith("-verbose=")) {
                input.setVerbose(Boolean.valueOf(arg.substring(9)).booleanValue());
            } else if (arg.startsWith("-create=")) {
                input.setCreate(Boolean.valueOf(arg.substring(8)).booleanValue());
            } else if (arg.startsWith("-drop=")) {
                input.setDrop(Boolean.valueOf(arg.substring(6)).booleanValue());
            } else if (arg.startsWith("-jdbcdriver=")) {
                input.setJdbcdriver(arg.substring(12));
                isFieldJdbcdriverProcessed = true;
            } else if (arg.startsWith("-jdbcurl=")) {
                input.setJdbcurl(arg.substring(9));
                isFieldJdbcurlProcessed = true;
            } else if (arg.startsWith("-jdbcuser=")) {
                input.setJdbcuser(arg.substring(10));
                isFieldJdbcuserProcessed = true;
            } else if (arg.startsWith("-jdbcpassword=")) {
                input.setJdbcpassword(arg.substring(14));
                isFieldJdbcpasswordProcessed = true;
            } else if (arg.equals("-?") || arg.equals("-help")) {
                usage();
                System.exit(END_SUCCESS);
            } else {
                System.out.println("BlancoFileManagerDbTableBatchProcess: 入力パラメータ[" + arg + "]は無視されました。");
                isNeedUsage = true;
            }
        }

        if (isNeedUsage) {
            usage();
        }

        if( isFieldJdbcdriverProcessed == false) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcdriver]に値が設定されていません。");
            System.exit(END_ILLEGAL_ARGUMENT_EXCEPTION);
        }
        if( isFieldJdbcurlProcessed == false) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcurl]に値が設定されていません。");
            System.exit(END_ILLEGAL_ARGUMENT_EXCEPTION);
        }
        if( isFieldJdbcuserProcessed == false) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcuser]に値が設定されていません。");
            System.exit(END_ILLEGAL_ARGUMENT_EXCEPTION);
        }
        if( isFieldJdbcpasswordProcessed == false) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcpassword]に値が設定されていません。");
            System.exit(END_ILLEGAL_ARGUMENT_EXCEPTION);
        }

        int retCode = batchProcess.execute(input);

        // 終了コードを戻します。
        // ※注意：System.exit()を呼び出している点に注意してください。
        System.exit(retCode);
    }

    /**
     * 具体的なバッチ処理内容を記述するためのメソッドです。
     *
     * このメソッドに実際の処理内容を記述します。
     *
     * @param input バッチ処理の入力パラメータ。
     * @return バッチ処理の終了コード。END_SUCCESS, END_ILLEGAL_ARGUMENT_EXCEPTION, END_IO_EXCEPTION, END_ERROR のいずれかの値を戻します。
     * @throws IOException 入出力例外が発生した場合。
     * @throws IllegalArgumentException 入力値に不正が見つかった場合。
     */
    public int process(final BlancoFileManagerDbTableProcessInput input) throws IOException, IllegalArgumentException {
        // 入力パラメータをチェックします。
        validateInput(input);

        // この箇所でコンパイルエラーが発生する場合、BlancoFileManagerDbTableProcessインタフェースを実装して blanco.filemanager.taskパッケージに BlancoFileManagerDbTableProcessImplクラスを作成することにより解決できる場合があります。
        final BlancoFileManagerDbTableProcess process = new BlancoFileManagerDbTableProcessImpl();

        // 処理の本体を実行します。
        final int retCode = process.execute(input);

        return retCode;
    }

    /**
     * クラスをインスタンス化してバッチを実行する際のエントリポイントです。
     *
     * このメソッドは下記の仕様を提供します。
     * <ul>
     * <li>メソッドの入力パラメータの内容チェック。
     * <li>IllegalArgumentException, RuntimeException, Errorなどの例外をcatchして戻り値へと変換。
     * </ul>
     *
     * @param input バッチ処理の入力パラメータ。
     * @return バッチ処理の終了コード。END_SUCCESS, END_ILLEGAL_ARGUMENT_EXCEPTION, END_IO_EXCEPTION, END_ERROR のいずれかの値を戻します。
     * @throws IllegalArgumentException 入力値に不正が見つかった場合。
     */
    public final int execute(final BlancoFileManagerDbTableProcessInput input) throws IllegalArgumentException {
        try {
            // バッチ処理の本体を実行します。
            int retCode = process(input);

            return retCode;
        } catch (IllegalArgumentException ex) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 入力例外が発生しました。バッチ処理を中断します。:" + ex.toString());
            // 入力異常終了。
            return END_ILLEGAL_ARGUMENT_EXCEPTION;
        } catch (IOException ex) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: 入出力例外が発生しました。バッチ処理を中断します。:" + ex.toString());
            // 入力異常終了。
            return END_IO_EXCEPTION;
        } catch (RuntimeException ex) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: ランタイム例外が発生しました。バッチ処理を中断します。:" + ex.toString());
            ex.printStackTrace();
            // 異常終了。
            return END_ERROR;
        } catch (Error er) {
            System.out.println("BlancoFileManagerDbTableBatchProcess: ランタイムエラーが発生しました。バッチ処理を中断します。:" + er.toString());
            er.printStackTrace();
            // 異常終了。
            return END_ERROR;
        }
    }

    /**
     * このバッチ処理クラスの使い方の説明を標準出力に示すためのメソッドです。
     */
    public static final void usage() {
        System.out.println("BlancoFileManagerDbTableBatchProcess: Usage:");
        System.out.println("  java blanco.filemanager.task.BlancoFileManagerDbTableBatchProcess -verbose=値1 -create=値2 -drop=値3 -jdbcdriver=値4 -jdbcurl=値5 -jdbcuser=値6 -jdbcpassword=値7");
        System.out.println("    -verbose");
        System.out.println("      説明[verboseモードで動作させるかどうか。]");
        System.out.println("      型[真偽]");
        System.out.println("      デフォルト値[false]");
        System.out.println("    -create");
        System.out.println("      説明[テーブルを作成する場合にセットします。]");
        System.out.println("      型[真偽]");
        System.out.println("      デフォルト値[true]");
        System.out.println("    -drop");
        System.out.println("      説明[テーブルを削除する場合にのみセットします。\n★これが true だと情報が削除される点に注意してください。]");
        System.out.println("      型[真偽]");
        System.out.println("      デフォルト値[false]");
        System.out.println("    -jdbcdriver");
        System.out.println("      説明[JDBC ドライバクラス名。\n例: org.apache.derby.jdbc.EmbeddedDriver]");
        System.out.println("      型[文字列]");
        System.out.println("      必須パラメータ");
        System.out.println("    -jdbcurl");
        System.out.println("      説明[JDBC 接続先 URL。\n例: jdbc:derby:tmp/derby.db;create=true]");
        System.out.println("      型[文字列]");
        System.out.println("      必須パラメータ");
        System.out.println("    -jdbcuser");
        System.out.println("      説明[JDBC 接続時のユーザー。\n例: user]");
        System.out.println("      型[文字列]");
        System.out.println("      必須パラメータ");
        System.out.println("    -jdbcpassword");
        System.out.println("      説明[JDBC 接続時のパスワード。\n例: password]");
        System.out.println("      型[文字列]");
        System.out.println("      必須パラメータ");
        System.out.println("    -? , -help");
        System.out.println("      説明[使い方を表示します。]");
    }

    /**
     * このバッチ処理クラスの入力パラメータの妥当性チェックを実施するためのメソッドです。
     *
     * @param input バッチ処理の入力パラメータ。
     * @throws IllegalArgumentException 入力値に不正が見つかった場合。
     */
    public void validateInput(final BlancoFileManagerDbTableProcessInput input) throws IllegalArgumentException {
        if (input == null) {
            throw new IllegalArgumentException("BlancoBatchProcessBatchProcess: 処理開始失敗。入力パラメータ[input]にnullが与えられました。");
        }
        if (input.getJdbcdriver() == null) {
            throw new IllegalArgumentException("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcdriver]に値が設定されていません。");
        }
        if (input.getJdbcurl() == null) {
            throw new IllegalArgumentException("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcurl]に値が設定されていません。");
        }
        if (input.getJdbcuser() == null) {
            throw new IllegalArgumentException("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcuser]に値が設定されていません。");
        }
        if (input.getJdbcpassword() == null) {
            throw new IllegalArgumentException("BlancoFileManagerDbTableBatchProcess: 処理開始失敗。入力パラメータ[input]の必須フィールド値[jdbcpassword]に値が設定されていません。");
        }
    }
}
